#include "CGameObject.h"

CGameObject::CGameObject(CSurfaceRotationCache *ImgIn, int Xin, int Yin, int Angle,int AngleInc, double Speed,int ZOrder,CSinCosTable *SinCosTableIn)
{
    Parent = NULL;
    Img = ImgIn;
    SinCosTable = SinCosTableIn;
    PX = Xin << Precission;
    PY = Yin << Precission;
    PSpeed = float2fix<Precission>(Speed);
    PAngle = Angle;
    PAngleInc = AngleInc;
    DisplayAngle = 0;
    UpdateAngle();
    RealImg = NULL;
    PKilled = false;
    CollisionRect.x = (Xin - (Img->GetSurfaceAngle(DisplayAngle)->w/2)) +  3;
    CollisionRect.y = (Yin - (Img->GetSurfaceAngle(DisplayAngle)->h/2)) +  3;
    CollisionRect.w = (Img->GetSurfaceAngle(DisplayAngle)->w) -  3;
    CollisionRect.x = (Img->GetSurfaceAngle(DisplayAngle)->h) -  3;
    PType = 0;
    PVisible = true;
    PCanMove = true;
    PZorder = ZOrder;
}

CGameObject::CGameObject(SDL_Surface *RealImgIn, int Xin, int Yin,int Angle,int AngleInc,double Speed,int ZOrder,CSinCosTable *SinCosTableIn)
{
    Parent = NULL;
    RealImg = RealImgIn;
    SinCosTable = SinCosTableIn;
    PX = Xin << Precission;
    PY = Yin << Precission;
    PSpeed = float2fix<Precission>(Speed);
    PAngle = Angle;
    PAngleInc = AngleInc;
    DisplayAngle = 0;
    UpdateAngle();
    PKilled = false;
    CollisionRect.x = (Xin - (RealImg->w/2)) +  3;
    CollisionRect.y = (Yin - (RealImg->h/2)) +  3;
    CollisionRect.w = (RealImg->w) -  3;
    CollisionRect.x = (RealImg->h) -  3;
    PType = 0;
    PVisible = true;
    PCanMove = true;
    PZorder = ZOrder;
}

CGameObject::~CGameObject()
{
}

void CGameObject::Draw(SDL_Surface *Dest)
{
    if(!PKilled && PVisible)
    {
        SDL_Rect Dst;
        SDL_Surface *Tmp;
        if(RealImg)
            Tmp = RealImg;
        else
            Tmp = Img->GetSurfaceAngle(DisplayAngle);

        Dst.x = (int)(fix2float<Precission>(PX)) - (Tmp->w/2);
        Dst.y = (int)(fix2float<Precission>(PY)) - (Tmp->h/2);
        Dst.w = Tmp->w;
        Dst.h = Tmp->h;
        SDL_BlitSurface(Tmp,NULL,Dest,&Dst);
    }
}

SDL_Surface *CGameObject::GetSurface()
{
     if(RealImg)
            return RealImg;
        else
            return Img->GetSurfaceAngle(DisplayAngle);
}

void CGameObject::UpdateAngle()
{
    if (PAngle < 0)
            PAngle = 360 + PAngle %360;
        else
            if (PAngle > 359)
                PAngle = PAngle%360;
    VelY = fixmul<Precission>(PSpeed,SinCosTable->GetSin(PAngle));
    VelX = fixmul<Precission>(PSpeed,SinCosTable->GetCos(PAngle));
}

void CGameObject::UpdateDisplayAngle()
{
    if (DisplayAngle < 0)
            DisplayAngle = 360 + DisplayAngle %360;
        else
            if (DisplayAngle > 359)
                DisplayAngle = DisplayAngle%360;
}

void CGameObject::Move()
{
    if(!PKilled)
    {
        if(PCanMove)
        {
            PX += VelX;
            PY += VelY;

            DisplayAngle += PAngleInc;
            UpdateDisplayAngle();
        }

        if(RealImg)
        {
            CollisionRect.x = (fix2float<Precission>(PX) - (RealImg->w/2))+3;
            CollisionRect.y = (fix2float<Precission>(PY) - (RealImg->h/2))+3;
            CollisionRect.w = (RealImg->w)-3;
            CollisionRect.h = (RealImg->h)-3;

            if(( (fix2float<Precission>(PX)) < 0) || (fix2float<Precission>(PX)  >= WINDOW_WIDTH) || ((fix2float<Precission>(PY) ) < 0) || (fix2float<Precission>(PY)  >= WINDOW_HEIGHT))
                PKilled = true;
        }
        else
        {
            CollisionRect.x = (fix2float<Precission>(PX) - (Img->GetSurfaceAngle(DisplayAngle)->w/2)) +  3;
            CollisionRect.y = (fix2float<Precission>(PY) - (Img->GetSurfaceAngle(DisplayAngle)->h/2)) +  3;
            CollisionRect.w = (Img->GetSurfaceAngle(DisplayAngle)->w) -  3;
            CollisionRect.h = (Img->GetSurfaceAngle(DisplayAngle)->h) -  3;

            if(( (fix2float<Precission>(PX) ) < 0) || (fix2float<Precission>(PX)  >= WINDOW_WIDTH) || ((fix2float<Precission>(PY) ) < 0) || (fix2float<Precission>(PY)  >= WINDOW_HEIGHT))
                PKilled = true;
        }

    }

}
