#ifndef CGAMEOBJECT_H
#define CGAMEOBJECT_H

#include <SDL.h>
#include "Functions.h"
#include "fixed_class.h"
#include "shooter.h"
#include "CSinCosTable.h"
#include "CSurfaceRotationCache.h"


using namespace fixedpoint;

class CWorldParts;

class CGameObject
{
    protected:
        SDL_Rect CollisionRect;
        int PType;
        CWorldParts *Parent;
        long PID;
        CSurfaceRotationCache *Img;
        SDL_Surface *RealImg;
        CSinCosTable *SinCosTable;
        long PX,PY,VelX,VelY,MapX,MapY,PSpeed;
        int PAngle,PAngleInc,DisplayAngle;
        void UpdateAngle();
        void UpdateDisplayAngle();
        bool PKilled, PCanMove, PVisible;
        int PZorder;
    public:

        CGameObject(CSurfaceRotationCache *ImgIn, int Xin, int Yin,int Angle,int AngleInc,double Speed,int Zorder,CSinCosTable *SinCosTableIn);
        CGameObject(SDL_Surface *RealImgIn, int Xin, int Yin,int Angle,int AngleInc,double Speed,int Zorder,CSinCosTable *SinCosTableIn);
        ~CGameObject();
        virtual void Draw(SDL_Surface *Dest);
        virtual void Move();
        void SetParent(CWorldParts *ParentIn) { Parent = ParentIn;}
        void SetID(long IDin) {PID = IDin;}
        long GetID() {return PID;}
        void SetType(long TypeIn) {PType = TypeIn;}
        int GetType() {return PType;}
        void SetKilled(bool KilledIn) {PKilled = KilledIn;}
        bool GetKilled() {return PKilled;}
        bool GetCanMove() {return PCanMove;}
        void SetCanMove(bool CanMove) { PCanMove = CanMove;}
        double GetX() { return fix2float<Precission>(PX);}
        double GetY() { return fix2float<Precission>(PY);}
        void SetX(int X) { PX = X << Precission;}
        void SetY(int Y) { PY = Y << Precission;}
        void SetSpeed(int Speed) { PSpeed = float2fix<Precission>(Speed);}
        double GetSpeed() { return fix2float<Precission>(PSpeed);}
        double GetVelX() { return fix2float<Precission>(VelX);}
        double GetVelY() { return fix2float<Precission>(VelY);}
        SDL_Rect *GetCollisionRect() {return &CollisionRect;}
        int GetZ() {return PZorder;}
        SDL_Surface *GetSurface();
};

#endif
