#include "CWorldParts.h"

CWorldParts::CWorldParts()
{
	ItemCount = 0;
	MapItemCount = 0;
	RemoveKilledCounter = 0;
	MapPosition = 0;
	MapInc=1;
	DisableSorting = false;
	BackgroundPos = 0;
	BackgroundCount = 0;
	LeveleditorMode = false;
}

void CWorldParts::SetLeveleditorMode(bool mode)
{
    LeveleditorMode = mode;
}

void CWorldParts::RemoveAll()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		delete Items[Teller];
		Items[Teller] = NULL;
	}
	ItemCount=0;
}

void CWorldParts::Sort()
{
	int Teller1,Teller2,Index;
	CGameObject *Part;
	if (!DisableSorting)
	{
		for (Teller1 = 1; Teller1 <ItemCount;Teller1++)
		{
			Index = Items[Teller1]->GetZ();
			Part = Items[Teller1];
			Teller2 = Teller1;
			while ((Teller2 > 0) && (Items[Teller2-1]->GetZ() > Index))
			{
				Items[Teller2] = Items[Teller2 - 1];
				Teller2--;
			}
			Items[Teller2] = Part;
		}
	}

}

void CWorldParts::AddPlayer(SDL_Surface* Img,int X, int Y)
{
    if(Player)
    {
        delete Player;
    }
    Player = new CPlayer(Img,X,Y,99,SinCosTable);
    this->Add((CGameObject *) Player);
}

void CWorldParts::PlayerMoveLeft(float inc)
{
    if(Player)
        Player->MoveLeft(inc);
}

void CWorldParts::PlayerMoveRight(float inc)
{
    if(Player)
        Player->MoveRight(inc);
}

void CWorldParts::PlayerMoveUp(float inc)
{
    if(Player)
        Player->MoveUp(inc);
}

void CWorldParts::PlayerMoveDown(float inc)
{
    if(Player)
        Player->MoveDown(inc);
}

void CWorldParts::PlayerShoot()
{
    if(Player)
        Player->Shoot();
}

void CWorldParts::RemoveKilled()
{
	int Teller1,Teller2;
	for (Teller1=0;Teller1<ItemCount;Teller1++)
	{
		if (Items[Teller1]->GetKilled())
		{
			delete Items[Teller1];
			for (Teller2=Teller1;Teller2<ItemCount-1;Teller2++)
				Items[Teller2] = Items[Teller2+1];
			ItemCount--;
			Teller1--;
		}
	}
}

void CWorldParts::DrawItemInfoFromPosAt(SDL_Surface *Surface,TTF_Font *font,int ItemX,int ItemY,int X,int Y,SDL_Color Color1)
{
    int Teller;
    char Text[1000];
    sprintf(Text,"");
    CEnemy *Enemy;
    for (Teller=0;Teller<ItemCount;Teller++)
	{
		if ((Items[Teller]->GetCollisionRect()->x <= ItemX) && (Items[Teller]->GetCollisionRect()->x + Items[Teller]->GetCollisionRect()->w >= ItemX) &&
            (Items[Teller]->GetCollisionRect()->y <= ItemY) && (Items[Teller]->GetCollisionRect()->y + Items[Teller]->GetCollisionRect()->h >= ItemY))
            {
                switch(Items[Teller]->GetType())
                {
                    case 3:
                        Enemy = (CEnemy* ) Items[Teller];
                        sprintf(Text,"%sMapX:%.0f\nMapY:%.0f\nHitPoints:%d\nMovementType:%d\nSpeed:%.2f\n\n",Text,Items[Teller]->GetX(),MapPosition-Items[Teller]->GetY(),Enemy->GetHitPoints(),Enemy->GetMovementType(),Items[Teller]->GetSpeed());
                        break;
                }
            }
	}
	WriteText(Surface,font,Text,strlen(Text),X,Y,2,Color1);
}

void CWorldParts::MoveAllDown()
{
    int Teller,Teller2;
    for(Teller2=0;Teller2<MapInc;Teller2++)
    {
        for (Teller=0;Teller<ItemCount;Teller++)
        {
            Items[Teller]->SetY(Items[Teller]->GetY() + 1);
            Items[Teller]->Move(); //For PKilled;
        }

        RemoveKilled();
        MapPosition++;
        BackgroundPos++;
        SpawnCheck(false);
    }

}

void CWorldParts::MoveAllUp()
{
    int Teller,Teller2;
    for(Teller2=0;Teller2<MapInc;Teller2++)
    {
        for (Teller=0;Teller<ItemCount;Teller++)
        {
            Items[Teller]->SetY(Items[Teller]->GetY() -1);
            Items[Teller]->Move(); //For PKilled;
        }
        if(MapPosition > 0)
        {
            RemoveKilled();
            MapPosition--;
            BackgroundPos--;
            SpawnCheck(true);
            if (BackgroundPos < 0)
                BackgroundPos = 0;

        }
        else
            RemoveKilled();


    }

}


void CWorldParts::SetCanMove(bool CanMove)
{
    int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		Items[Teller]->SetCanMove(CanMove);
	}
}

void CWorldParts::SpawnCheck(bool ReverseAlso)
{
    int Teller;
    CGameObject *obj;
    for(Teller = 0; Teller < MapItemCount; Teller++)
    {
        if(!ReverseAlso)
        if((MapItems[Teller].PY == MapPosition))
        {
            switch(MapItems[Teller].PType)
            {

                case 3: //Enemy
                    switch(MapItems[Teller].PImgType)
                    {
                        case 1:
                            obj = (CGameObject *) new CEnemy(Triangle,MapItems[Teller].PX,0 ,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 2:
                            obj = (CGameObject *) new CEnemy(Square,MapItems[Teller].PX,0 ,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 3:
                            obj = (CGameObject *) new CEnemy(Circle,MapItems[Teller].PX,0 ,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 4:
                            obj = (CGameObject *) new CEnemy(Star1,MapItems[Teller].PX,0 ,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 5:
                            obj = (CGameObject *) new CEnemy(Star2,MapItems[Teller].PX,0 ,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 6:
                            obj = (CGameObject *) new CEnemy(Star3,MapItems[Teller].PX,0 ,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 7:
                            obj = (CGameObject *) new CEnemy(Polygon,MapItems[Teller].PX,0 ,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                    }
                    break;
            }

        }

        if(ReverseAlso)
        if((MapItems[Teller].PY == MapPosition  - (WINDOW_HEIGHT -1)))
        {
            switch(MapItems[Teller].PType)
            {

                case 3: //Enemy
                    switch(MapItems[Teller].PImgType)
                    {
                        case 1:
                            obj = (CGameObject *) new CEnemy(Triangle,MapItems[Teller].PX,WINDOW_HEIGHT -1,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 2:
                            obj = (CGameObject *) new CEnemy(Square,MapItems[Teller].PX,WINDOW_HEIGHT -1,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 3:
                            obj = (CGameObject *) new CEnemy(Circle,MapItems[Teller].PX,WINDOW_HEIGHT -1,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 4:
                            obj = (CGameObject *) new CEnemy(Star1,MapItems[Teller].PX,WINDOW_HEIGHT -1,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 5:
                            obj = (CGameObject *) new CEnemy(Star2,MapItems[Teller].PX,WINDOW_HEIGHT -1,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 6:
                            obj = (CGameObject *) new CEnemy(Star3,MapItems[Teller].PX,WINDOW_HEIGHT -1,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                        case 7:
                            obj = (CGameObject *) new CEnemy(Polygon,MapItems[Teller].PX,WINDOW_HEIGHT -1,MapItems[Teller].PSubType,MapItems[Teller].PSpeed,MapItems[Teller].HitPoints,10,SinCosTable);
                            obj->SetCanMove(!LeveleditorMode);
                            this->Add(obj);
                            break;
                    }
                    break;
            }

        }
    }

}

void CWorldParts::DrawBackground(SDL_Surface *Dest)
{
   SDL_Rect Rect1,Rect2;

    if(BackgroundPos + BackgroundHeight < (BackgroundCount *BackgroundHeight))
    {
        Rect1.x = 0;
        Rect1.w = 320;
        Rect1.y = (BackgroundPos % BackgroundHeight) - BackgroundHeight-60;
        Rect1.h = WINDOW_HEIGHT+60;
       switch(Backgrounds[(BackgroundPos / BackgroundHeight)+1])
       {
            case 1 :
                SDL_BlitSurface(IMGBackgroundEarth,NULL,Dest,&Rect1);break;
            case 2 :
                SDL_BlitSurface(IMGBackgroundGrass,NULL,Dest,&Rect1);break;
            case 3 :
                SDL_BlitSurface(IMGBackgroundWater,NULL,Dest,&Rect1);break;
            case 4 :
                SDL_BlitSurface(IMGBackgroundEarthGrass,NULL,Dest,&Rect1);break;
            case 5 :
                SDL_BlitSurface(IMGBackgroundGrassEarth,NULL,Dest,&Rect1);break;
            case 6 :
                SDL_BlitSurface(IMGBackgroundGrassWater,NULL,Dest,&Rect1);break;
            case 7 :
                SDL_BlitSurface(IMGBackgroundWaterGrass,NULL,Dest,&Rect1);break;
            case 8 :
                SDL_BlitSurface(IMGBackgroundEarthPond,NULL,Dest,&Rect1);break;

       }
   }
   else
        if(!LeveleditorMode)
        {
            Rect1.x = 0;
            Rect1.w = 320;
            Rect1.y = (BackgroundPos % BackgroundHeight) - BackgroundHeight-60;
            Rect1.h = WINDOW_HEIGHT+60;
           switch(Backgrounds[BackgroundCount-1])
           {
                case 1 :
                    SDL_BlitSurface(IMGBackgroundEarth,NULL,Dest,&Rect1);break;
                case 2 :
                    SDL_BlitSurface(IMGBackgroundGrass,NULL,Dest,&Rect1);break;
                case 3 :
                    SDL_BlitSurface(IMGBackgroundWater,NULL,Dest,&Rect1);break;
                case 4 :
                    SDL_BlitSurface(IMGBackgroundEarthGrass,NULL,Dest,&Rect1);break;
                case 5 :
                    SDL_BlitSurface(IMGBackgroundGrassEarth,NULL,Dest,&Rect1);break;
                case 6 :
                    SDL_BlitSurface(IMGBackgroundGrassWater,NULL,Dest,&Rect1);break;
                case 7 :
                    SDL_BlitSurface(IMGBackgroundWaterGrass,NULL,Dest,&Rect1);break;
                case 8 :
                    SDL_BlitSurface(IMGBackgroundEarthPond,NULL,Dest,&Rect1);break;

           }
        }
        else
            SDL_FillRect(Dest,NULL,0);


    if(BackgroundPos < BackgroundHeight * BackgroundCount)
    {
        Rect1.x = 0;
        Rect1.w = 320;
        Rect1.y = (BackgroundPos % (BackgroundHeight))-60;
        Rect1.h = WINDOW_HEIGHT+60;

        switch(Backgrounds[BackgroundPos / BackgroundHeight])
        {
            case 1 :
                SDL_BlitSurface(IMGBackgroundEarth,NULL,Dest,&Rect1);break;
            case 2 :
                SDL_BlitSurface(IMGBackgroundGrass,NULL,Dest,&Rect1);break;
            case 3 :
                SDL_BlitSurface(IMGBackgroundWater,NULL,Dest,&Rect1);break;
            case 4 :
                SDL_BlitSurface(IMGBackgroundEarthGrass,NULL,Dest,&Rect1);break;
            case 5 :
                SDL_BlitSurface(IMGBackgroundGrassEarth,NULL,Dest,&Rect1);break;
            case 6 :
                SDL_BlitSurface(IMGBackgroundGrassWater,NULL,Dest,&Rect1);break;
            case 7 :
                SDL_BlitSurface(IMGBackgroundWaterGrass,NULL,Dest,&Rect1);break;
            case 8 :
                SDL_BlitSurface(IMGBackgroundEarthPond,NULL,Dest,&Rect1);break;

        }

    }
    else
        if(!LeveleditorMode)
            BackgroundPos = (BackgroundCount -2) * BackgroundHeight;

    if(!LeveleditorMode)
        BackgroundPos +=   MapInc;
}

void CWorldParts::AddBackGroundPart(int Type)
{
    if( BackgroundCount < MaxBackgrounds )
	{
	    Backgrounds[BackgroundCount] = Type;
	    BackgroundCount++;
	}
}

void CWorldParts::AddMapItem(int MapX,long MapY,int ImgType, int Type, int SubType, double Speed, int HitPoints)
{
    if( MapItemCount < MaxMapItems )
	{
	    MapItems[MapItemCount].PX = MapX;
	    MapItems[MapItemCount].PY = MapY;
	    MapItems[MapItemCount].PType = Type;
	    MapItems[MapItemCount].PSubType = SubType;
	    MapItems[MapItemCount].PImgType = ImgType;
	    MapItems[MapItemCount].PSpeed = Speed;
	    MapItems[MapItemCount].HitPoints = HitPoints;
		MapItemCount++;
	}
}

void CWorldParts::Add(CGameObject *WorldPart)
{
	if( ItemCount < MaxObjects )
	{
	    WorldPart->SetParent(this);
		Items[ItemCount] = WorldPart;
		ItemCount++;
        Sort();
	}
}

void CWorldParts::Move()
{

	int Teller,Teller2;
	for (Teller2 = 0;Teller2<MapInc;Teller2++)
    {
        for (Teller=0;Teller<ItemCount;Teller++)
            Items[Teller]->Move();

        RemoveKilled();
        MapPosition++;
        SpawnCheck(false);
    }

}

void CWorldParts::Draw(SDL_Surface *Surface)
{
	int Teller;
    DrawBackground(Surface);
	for (Teller=0;Teller<ItemCount;Teller++)
		Items[Teller]->Draw(Surface);

}

CWorldParts::~CWorldParts()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		delete Items[Teller];
		Items[Teller] = NULL;
	}
	//printf("Deleted worldparts\n");
}

