#ifndef CWORLDPARTS_H
#define CWORLDPARTS_H

#include <SDL_ttf.h>
#include "CGameObject.h"
#include "CSurfaceRotationCache.h"
#include "CAnimationObject.h"
#include "CEnemy.h"
#include "CSinCosTable.h"
#include "Functions.h"
#include "CPlayer.h"

const int   MaxObjects = 1000,
            RemoveKilledTimeOut = 0,
            MaxMapItems = 1000,
            MaxBackgrounds = 100,
            BackgroundHeight = 300;


extern SDL_Surface  *IMGBackgroundStar1,*IMGBackgroundStar2, *IMGAnimStar1, *IMGAnimStar2,
                    *IMGBackgroundGrass,*IMGBackgroundWater,*IMGBackgroundEarth,*IMGBackgroundWaterGrass,*IMGBackgroundGrassWater, *IMGBackgroundGrassEarth,*IMGBackgroundEarthGrass,
                    *IMGBackgroundEarthPond,
                    *Screen;
extern CSurfaceRotationCache *Star1,*Star2,*Star3, *Square, *Circle, *Polygon, *Triangle;
extern CSinCosTable *SinCosTable;


struct MapItem
{
    long PY;
    int PX;
    int PImgType,PType,PSubType,HitPoints;
    double PSpeed;
};

class CPlayer;

class CWorldParts
{

    private:
        CPlayer *Player;
        SDL_Surface *TmpBackgroundSurface;
        int RemoveKilledCounter;
        void SpawnCheck(bool ReverseAlso);
        void Sort();
        bool DisableSorting;
        int BackgroundCount;
        long BackgroundPos;
        int Backgrounds[MaxBackgrounds];
        bool LeveleditorMode;
    public:
        void AddPlayer(SDL_Surface* Img,int X, int Y);
        void PlayerMoveLeft(float inc);
        void PlayerMoveRight(float inc);
        void PlayerMoveUp(float inc);
        void PlayerMoveDown(float inc);
        void PlayerShoot();
        void SetLeveleditorMode(bool mode);
        void DrawBackground(SDL_Surface *Dest);
        CGameObject *Items[MaxObjects];
        MapItem MapItems[MaxMapItems];
        int MapInc;
        int ItemCount;
        int MapItemCount;
        long MapPosition;
        CWorldParts();
        void AddBackGroundPart(int Type);
        void Add(CGameObject *WorldPart);
        void AddMapItem(int MapX,long MapY, int ImgType, int Type, int SubType, double Speed, int HitPoints);
        void Move();
        void HistoryAdd();
        void HistoryGoBack();
        void Draw(SDL_Surface *Surface);
        void RemoveAll();
        void RemoveKilled();
        void SetCanMove(bool CanMove);
        void MoveAllDown();
        void MoveAllUp();
        void DrawItemInfoFromPosAt(SDL_Surface *Surface,TTF_Font *font,int ItemX,int ItemY,int X,int Y,SDL_Color Color1);
    //	void Save(char *Filename);
    //	void Load(char *Filename);
        ~CWorldParts();
};

#endif
